@@ -6,6 +6,7 @@ DEFAULT_FLOAT_WM_TYPES = set([
     'notification',
     'toolbar',
     'splash',
+    'dialog',
 ])
 
 
@@ -112,7 +113,9 @@ class Floating(Layout):
         if self.clients:
             return self.clients[-1]
 
-    def focus_prev(self, win):
+    def focus_previous(self, win):
+        if win not in self.clients:
+            return
         idx = self.clients.index(win)
         if idx > 0:
             return self.clients[idx - 1]
@@ -151,15 +154,26 @@ class Floating(Layout):
 
     def add(self, client):
         self.clients.append(client)
+        self.focused = client
 
     def remove(self, client):
         if client not in self.clients:
             return
-        res = self.focus_next(client)
+        self.focused = self.focus_next(client)
         self.clients.remove(client)
-        return res
+        return self.focused
 
     def info(self):
         d = Layout.info(self)
         d["clients"] = [x.name for x in self.clients]
         return d
+
+    def cmd_next(self):
+        client = self.focus_next(self.focused) or \
+            self.focus_first()
+        self.group.focus(client, False)
+
+    def cmd_previous(self):
+        client = self.focus_previous(self.focused) or \
+            self.focus_last()
+        self.group.focus(client, False)
