@@ -110,6 +110,7 @@ class MonadTall(SingleWindow):
         ("border_normal", "#000000", "Border colour for un-focused winows."),
         ("border_width", 2, "Border width."),
         ("name", "xmonad-tall", "Name of this layout."),
+        ("margin", 0, "Margin of the layout"),
     ]
 
     def __init__(self, ratio=_med_ratio, align=_left, change_ratio=.05,
@@ -260,7 +261,8 @@ class MonadTall(SingleWindow):
                 self.group.screen.dwidth,
                 self.group.screen.dheight,
                 0,
-                px
+                px,
+                margin=self.margin,
             )
             client.unhide()
             return
@@ -313,7 +315,8 @@ class MonadTall(SingleWindow):
                 width,
                 height - 2 * self.border_width,
                 self.border_width,
-                px
+                px,
+                margin=self.margin,
             )
             client.unhide()
         else:
@@ -324,7 +327,8 @@ class MonadTall(SingleWindow):
                 width,
                 self.group.screen.dheight - 2 * self.border_width,
                 self.border_width,
-                px
+                px,
+                margin=self.margin,
             )
             client.unhide()
 
@@ -591,6 +595,40 @@ class MonadTall(SingleWindow):
         self.relative_sizes[self.focused - 1] -= \
             self._get_relative_size_from_absolute(change)
 
+    def focus_first(self):
+        if self.clients:
+            return self.clients[0]
+
+    def focus_last(self):
+        if self.clients:
+            return self.clients[-1]
+
+    def focus_next(self, window):
+        if not self.clients:
+            return
+        if self.focused != self.clients.index(window):
+            self.focus(window)
+        if self.focused + 1 < len(self.clients):
+            return self.clients[self.focused + 1]
+
+    def focus_previous(self, window):
+        if not self.clients:
+            return
+        if self.focused != self.clients.index(window):
+            self.focus(window)
+        if self.focused > 0:
+            return self.clients[self.focused - 1]
+
+    def cmd_next(self):
+        client = self.focus_next(self.clients[self.focused]) or \
+            self.focus_first()
+        self.group.focus(client, False)
+
+    def cmd_previous(self):
+        client = self.focus_previous(self.clients[self.focused]) or \
+            self.focus_last()
+        self.group.focus(client, False)
+
     def cmd_shrink(self):
         """
         Will shrink the currently focused client reducing the
