@@ -1,4 +1,4 @@
-from libqtile.config import Key, Screen, Group
+from libqtile.config import Key, Screen, Group, Drag, Click
 from libqtile.command import lazy
 from libqtile import layout, bar, widget
 
@@ -48,23 +48,16 @@ keys = [
     Key([mod], "Return", lazy.spawn("xterm")),
 
     # Toggle between different layouts as defined below
-    Key([mod], "Tab",    lazy.nextlayout()),
-    Key([mod], "w",      lazy.window.kill()),
+    Key([mod], "Tab", lazy.nextlayout()),
+    Key([mod], "w", lazy.window.kill()),
 
     Key([mod, "control"], "r", lazy.restart()),
+    Key([mod, "control"], "q", lazy.shutdown()),
     Key([mod], "r", lazy.spawncmd()),
 ]
 
-groups = [
-    Group("a"),
-    Group("s"),
-    Group("d"),
-    Group("f"),
-    Group("u"),
-    Group("i"),
-    Group("o"),
-    Group("p"),
-]
+groups = [Group(i) for i in "asdfuiop"]
+
 for i in groups:
     # mod1 + letter of group = switch to group
     keys.append(
@@ -76,14 +69,17 @@ for i in groups:
         Key([mod, "shift"], i.name, lazy.window.togroup(i.name))
     )
 
-dgroups_key_binder = None
-dgroups_app_rules = []
-
 layouts = [
     layout.Max(),
-    layout.Stack(stacks=2)
+    layout.Stack(num_stacks=2)
 ]
 
+widget_defaults = dict(
+    font='Arial',
+    fontsize=16,
+    padding=3,
+)
+
 screens = [
     Screen(
         bottom=bar.Bar(
@@ -93,18 +89,28 @@ screens = [
                 widget.WindowName(),
                 widget.TextBox("default config", name="default"),
                 widget.Systray(),
-                widget.Clock('%Y-%m-%d %a %I:%M %p'),
+                widget.Clock(format='%Y-%m-%d %a %I:%M %p'),
             ],
             30,
         ),
     ),
 ]
 
+# Drag floating layouts.
+mouse = [
+    Drag([mod], "Button1", lazy.window.set_position_floating(),
+        start=lazy.window.get_position()),
+    Drag([mod], "Button3", lazy.window.set_size_floating(),
+        start=lazy.window.get_size()),
+    Click([mod], "Button2", lazy.window.bring_to_front())
+]
+
+dgroups_key_binder = None
+dgroups_app_rules = []
 main = None
 follow_mouse_focus = True
 bring_front_click = False
 cursor_warp = False
 floating_layout = layout.Floating()
-mouse = ()
 auto_fullscreen = True
-widget_defaults = {}
+wmname = "qtile"
