@@ -1,7 +1,12 @@
+# https://bitbucket.org/tarek/flake8/issue/141/improve-flake8-statement-to-ignore
+# is annoying, so we ignore libqtile/widget/__init__.py completely
+# flake8: noqa
+
 from backlight import Backlight
 from battery import Battery, BatteryIcon
 from clock import Clock
 from currentlayout import CurrentLayout
+from debuginfo import DebugInfo
 from graph import CPUGraph, MemoryGraph, SwapGraph, NetGraph, HDDGraph, HDDBusyGraph
 from groupbox import AGroupBox, GroupBox
 from maildir import Maildir
@@ -20,6 +25,13 @@ from keyboardlayout import KeyboardLayout
 from df import DF
 from image import Image
 from gmail_checker import GmailChecker
+from clipboard import Clipboard
+from countdown import Countdown
+
+try:
+    from launchbar import LaunchBar
+except ImportError:
+    pass
 
 from tasklist import TaskList
 
@@ -33,6 +45,11 @@ try:
 except ImportError:
     pass
 
+try:
+    from mpris2widget import Mpris2
+except ImportError:
+    pass
+
 try:
     from mpdwidget import Mpd
 except ImportError:
@@ -45,7 +62,6 @@ except ImportError:
     # Requires Python >= 2.6 or simplejson
     pass
 from pacman import Pacman
-from sensors import ThermalSensor
 try:
     from wlan import Wlan
 except ImportError:
