@@ -1,6 +1,4 @@
-import cairo
 import os
-from libqtile import bar
 import base
 
 BACKLIGHT_DIR = '/sys/class/backlight'
@@ -8,7 +6,7 @@ BACKLIGHT_DIR = '/sys/class/backlight'
 FORMAT = '{percent: 2.0%}'
 
 
-class Backlight(base._TextBox):
+class Backlight(base.InLoopPollText):
     """
         A simple widget to show the current brightness of a monitor.
     """
@@ -29,13 +27,12 @@ class Backlight(base._TextBox):
             'Name of file with the '
             'maximum brightness in /sys/class/backlight/backlight_name'
         ),
-        ('update_delay', .2, 'The delay in seconds between updates'),
+        ('update_interval', .2, 'The delay in seconds between updates'),
     ]
 
     def __init__(self, **config):
-        base._TextBox.__init__(self, "LIGHT", bar.CALCULATED, **config)
+        base.InLoopPollText.__init__(self, **config)
         self.add_defaults(Backlight.defaults)
-        self.timeout_add(self.update_delay, self.update)
 
     def _load_file(self, name):
         try:
@@ -57,18 +54,10 @@ class Backlight(base._TextBox):
             return False
         return info
 
-    def _get_text(self):
+    def poll(self):
         info = self._get_info()
         if info is False:
             return 'Error'
 
         percent = info['brightness'] / info['max']
         return FORMAT.format(percent=percent)
-
-    def update(self):
-        if self.configured:
-            ntext = self._get_text()
-            if ntext != self.text:
-                self.text = ntext
-                self.bar.draw()
-        return True
